@@ -13,4 +13,6 @@
 
 
 """
-from zeep.wsdl.wsdl import Document  # noqa
+from zeep.wsdl.wsdl import Document
+
+__all__ = ["Document"]
